@@ -1,4 +1,5 @@
 from fastapi import FastAPI
+import traceback
 from fastapi.exceptions import RequestValidationError
 from starlette.exceptions import HTTPException as StarletteHTTPException
 from starlette.requests import Request
@@ -23,7 +24,7 @@ async def http_exception_handler(
     """
     fields = getattr(exc, "fields", [])
     data = {
-        "code": getattr(exc, "error_code", exc.status_code),
+        "code": getattr(exc, "code", "Error"),
         "detail": getattr(exc, "message", exc.detail),
         "fields": fields,
     }
@@ -45,10 +46,24 @@ async def validation_exception_handler(
     """
     LOG.warning(f"RequestValidationError Caught: {str(exc)}")
     status_code = getattr(exc, "status_code", 422)
-    data = {
-        "code": getattr(exc, "error_code", status_code),
-        "detail": str(exc),
-    }
+    errors = []
+    details = str(exc)
+    try:
+        for error in exc.errors():
+            errors.append(
+                {
+                    "area": error.get("loc", ("", ""))[0],
+                    "variable": error.get("loc", ("", ""))[1],
+                    "message": error.get("msg"),
+                    "type": error.get("type"),
+                }
+            )
+        details = "One or more fields has failed validation."
+    except Exception as err:
+        LOG.error(f"Problem creating validation error response. {err}")
+
+    # FIXME make validation errors better
+    data = {"code": "Validation Error", "detail": details, "errors": errors}
     return UJSONResponse(data, status_code=status_code)
 
 
@@ -66,33 +81,47 @@ async def pydantic_validation_exception_handler(
     :return: UJSONResponse with newly formatted error data
     """
     LOG.error(f"Unhandled Pydantic ValidationError Caught: {str(exc)}")
-    status_code = getattr(exc, "status_code", 500)
+    errors = []
+    try:
+        for error in exc.errors():
+            errors.append(
+                {
+                    "area": error.get("loc", ("", ""))[0],
+                    "variable": error.get("loc", ("", ""))[1],
+                    "message": error.get("msg"),
+                    "type": error.get("type"),
+                }
+            )
+    except Exception as err:
+        LOG.error(f"Problem creating validation error response. {err}")
+
     data = {
-        "code": getattr(exc, "error_code", status_code),
-        "detail": f"A domain object has failed validation this is likely due to changes in the database "
-        + f"structure. Error: {exc}",
+        "code": "Server Validation Error",
+        "detail": f"A domain object has failed validation this is likely due to changes in the model or "
+        + f"database structure. Creating creating a new dataset may solve the issue.",
+        "error": str(exc),
     }
-    return UJSONResponse(data, status_code=status_code)
+    return UJSONResponse(data, status_code=500)
 
 
 async def assert_exception(request: Request, exc: AssertionError) -> UJSONResponse:
     LOG.warning(f"Assert exception caught: {str(exc)}")
-    status_code = getattr(exc, "status_code", 400)
-    data = {
-        "code": status_code,
-        "detail": getattr(exc, "message", str(exc)),
-    }
-    return UJSONResponse(data, status_code=status_code)
+    data = {"code": "Assertion Error", "detail": str(exc)}
+    return UJSONResponse(
+        data, status_code=400, headers={"Access-Control-Allow-Origin": "*"}
+    )
 
 
 async def default_exception(request: Request, exc: Exception) -> UJSONResponse:
     LOG.error(f"Internal exception caught: {str(exc)}")
-    status_code = getattr(exc, "status_code", 500)
     data = {
-        "code": status_code,
-        "detail": getattr(exc, "message", "Internal server error"),
+        "code": "Internal Server Error",
+        "detail": str(exc),
+        "trace": traceback.format_exc(),
     }
-    return UJSONResponse(data, status_code=status_code)
+    return UJSONResponse(
+        data, status_code=500, headers={"Access-Control-Allow-Origin": "*"}
+    )
 
 
 def setup_exception_handlers(app: FastAPI) -> None:
